Class: Mixlib::Install::Backend::PackageRouter

Inherits:
Base
  • Object
show all
Defined in:
lib/mixlib/install/backend/package_router.rb

Constant Summary collapse

COMPAT_DOWNLOAD_URL_ENDPOINT =
"http://packages.chef.io".freeze

Instance Attribute Summary

Attributes inherited from Base

#options

Instance Method Summary collapse

Methods inherited from Base

#filter_artifacts, #info, #initialize, #normalize_platform, #platform_filters_available?, #windows_artifact_fixup!

Constructor Details

This class inherits a constructor from Mixlib::Install::Backend::Base

Instance Method Details

#artifacts_for_version(version) ⇒ Array<ArtifactInfo>

Get artifacts for a given version, channel and product_name

Returns:

  • (Array<ArtifactInfo>)

    Array of info about found artifacts



134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
# File 'lib/mixlib/install/backend/package_router.rb', line 134

def artifacts_for_version(version)
  begin
    if use_licensed_api?
      # Commercial/trial APIs use the packages endpoint which returns metadata for all platforms
      query = "v=#{version}"
      packages_hash = get("/#{options.channel}/#{omnibus_project}/packages?#{query}")
      # Response structure differs between products:
      # - For chef-ice: platform -> architecture -> package_manager -> package_info
      # - For other products: platform -> platform_version -> architecture -> package_info
      results = []
      if omnibus_project == "chef-ice"
        # chef-ice structure: platform -> architecture -> package_manager -> package_info
        packages_hash.each do |platform, architectures|
          architectures.each do |arch, package_managers|
            package_managers.each do |pm, pkg_info|
              results << {
                "omnibus.version" => pkg_info["version"],
                "omnibus.platform" => platform,
                "omnibus.platform_version" => "",
                "omnibus.architecture" => arch,
                "omnibus.project" => omnibus_project,
                "omnibus.license" => "Apache-2.0",
                "omnibus.sha256" => pkg_info["sha256"],
                "omnibus.sha1" => pkg_info.fetch("sha1", ""),
                "omnibus.md5" => pkg_info.fetch("md5", ""),
                "omnibus.package_manager" => pm,
              }
            end
          end
        end
      else
        # Standard structure: platform -> platform_version -> architecture -> package_info
        packages_hash.each do |platform, platform_versions|
          platform_versions.each do |platform_version, architectures|
            architectures.each do |arch, pkg_info|
              results << {
                "omnibus.version" => pkg_info["version"],
                "omnibus.platform" => platform,
                "omnibus.platform_version" => platform_version,
                "omnibus.architecture" => arch,
                "omnibus.project" => omnibus_project,
                "omnibus.license" => "Apache-2.0",
                "omnibus.sha256" => pkg_info["sha256"],
                "omnibus.sha1" => pkg_info.fetch("sha1", ""),
                "omnibus.md5" => pkg_info.fetch("md5", ""),
              }
            end
          end
        end
      end
    else
      results = get("/api/v1/#{options.channel}/#{omnibus_project}/#{version}/artifacts")["results"]
      # Merge artifactory properties to a flat Hash
      results.collect! do |result|
        {
          "filename" => result["name"],
        }.merge(
          map_properties(result["properties"])
        )
      end
    end
  rescue Net::HTTPServerException => e
    if e.message.match?(/404/)
      return []
    else
      raise e
    end
  end

  # Convert results to build records
  results.map { |a| create_artifact(a) }
end

#available_artifactsArray<ArtifactInfo>

Create filtered list of artifacts

channel, product name, and product version.

Returns:

  • (Array<ArtifactInfo>)

    list of artifacts for the configured



40
41
42
43
44
45
46
47
# File 'lib/mixlib/install/backend/package_router.rb', line 40

def available_artifacts
  artifacts = if options.latest_version? || options.partial_version?
                latest_version
              else
                artifacts_for_version(options.product_version)
              end
  windows_artifact_fixup!(artifacts)
end

#available_versionsArray<String>

Gets available versions from Artifactory via AQL. Returning simply the list of versions.

Returns:

  • (Array<String>)

    Array of available versions



54
55
56
57
58
59
60
# File 'lib/mixlib/install/backend/package_router.rb', line 54

def available_versions
  # We are only including a single property, version and that exists
  # under the properties in the following structure:
  # "properties" => [ {"key"=>"omnibus.version", "value"=>"12.13.3"} ]
  ver_list = versions.map { |i| Mixlib::Versioning.parse(extract_version_from_response(i)) }.sort
  ver_list.uniq.map(&:to_s)
end

#create_artifact(artifact_map) ⇒ Object



245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
# File 'lib/mixlib/install/backend/package_router.rb', line 245

def create_artifact(artifact_map)
  # set normalized platform and platform version
  platform, platform_version = normalize_platform(
    artifact_map["omnibus.platform"],
    artifact_map["omnibus.platform_version"]
  )

  # create the standardized file path
  chef_standard_path = generate_chef_standard_path(
    options.channel,
    artifact_map["omnibus.project"],
    artifact_map["omnibus.version"],
    platform,
    platform_version,
    artifact_map["filename"]
  )

  if options.include_metadata?
    # retrieve the metadata using the standardized path
    begin
       = get("#{chef_standard_path}.metadata.json")
      license_content = ["license_content"]
      software_dependencies = .fetch("version_manifest", {})
                                .fetch("software", nil)
    rescue Net::HTTPServerException => e
      if e.message.match?(/404/)
        license_content, software_dependencies = nil
      else
        raise e
      end
    end
  else
    license_content, software_dependencies = nil
  end

  # create the download path with the correct endpoint
  if use_licensed_api?
    # Commercial/trial APIs use the download endpoint with query parameters
    # Construct platform parameters
    p_param = platform
    pv_param = platform_version
    m_param = Util.normalize_architecture(artifact_map["omnibus.architecture"])
    v_param = artifact_map["omnibus.version"]

    # For chef-ice, use normalized platform names and add package manager parameter
    if omnibus_project == "chef-ice"
      p_param = Util.normalize_platform_for_commercial(platform)
      # Use package_manager from artifact_map if available, otherwise determine it
      pm_param = artifact_map.fetch("omnibus.package_manager", Util.determine_package_manager(options.platform))
      download_url = "#{endpoint}/#{options.channel}/#{omnibus_project}/download?v=#{v_param}&license_id=#{options.license_id}&m=#{m_param}&p=#{p_param}&pm=#{pm_param}"
    else
      download_url = "#{endpoint}/#{options.channel}/#{omnibus_project}/download?p=#{p_param}&pv=#{pv_param}&m=#{m_param}&v=#{v_param}&license_id=#{options.license_id}"
    end
  else
    base_url = if use_compat_download_url_endpoint?(platform, platform_version)
                 COMPAT_DOWNLOAD_URL_ENDPOINT
               else
                 endpoint
               end
    download_url = "#{base_url}/#{chef_standard_path}"
  end

  ArtifactInfo.new(
    architecture:          Util.normalize_architecture(artifact_map["omnibus.architecture"]),
    license:               artifact_map["omnibus.license"],
    license_content:       license_content,
    md5:                   artifact_map["omnibus.md5"],
    platform:              platform,
    platform_version:      platform_version,
    product_description:   product_description,
    product_name:          options.product_name,
    sha1:                  artifact_map["omnibus.sha1"],
    sha256:                artifact_map["omnibus.sha256"],
    software_dependencies: software_dependencies,
    url:                   download_url,
    version:               artifact_map["omnibus.version"]
  )
end

#create_http_request(full_path) ⇒ Object



237
238
239
240
241
242
243
# File 'lib/mixlib/install/backend/package_router.rb', line 237

def create_http_request(full_path)
  request = Net::HTTP::Get.new(full_path)

  request.add_field("User-Agent", Util.user_agent_string(options.user_agent_headers))

  request
end

#endpointObject

Public API detection methods for testing



341
342
343
344
345
346
347
348
349
# File 'lib/mixlib/install/backend/package_router.rb', line 341

def endpoint
  @endpoint ||= if use_trial_api?
                  Mixlib::Install::Dist::TRIAL_API_ENDPOINT
                elsif use_commercial_api?
                  Mixlib::Install::Dist::COMMERCIAL_API_ENDPOINT
                else
                  PRODUCT_MATRIX.lookup(options.product_name, options.product_version).api_url
                end
end

#extract_version_from_response(response) ⇒ Object



126
127
128
# File 'lib/mixlib/install/backend/package_router.rb', line 126

def extract_version_from_response(response)
  response["properties"].find { |item| item["key"] == "omnibus.version" }["value"]
end

#get(url) ⇒ Object

GET request



210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
# File 'lib/mixlib/install/backend/package_router.rb', line 210

def get(url)
  uri = URI.parse(endpoint)
  http = Net::HTTP.new(uri.host, uri.port)
  http.use_ssl = (uri.scheme == "https")
  full_path = File.join(uri.path, url)

  # Add license_id as query parameter if using commercial or trial API
  if use_licensed_api?
    separator = full_path.include?("?") ? "&" : "?"
    full_path = "#{full_path}#{separator}license_id=#{options.license_id}"
  end

  res = http.request(create_http_request(full_path))
  res.value
  JSON.parse(res.body)
rescue Net::HTTPClientError, Net::HTTPServerError => e
  # Provide helpful error messages for licensed API failures
  if use_trial_api?
    if options.channel != :stable || (options.product_version != :latest && options.product_version.to_sym != :latest)
      raise "Trial API only supports stable channel and latest version. " \
            "Current settings: channel=#{options.channel}, version=#{options.product_version}. " \
            "Error: #{e.message}"
    end
  end
  raise e
end

#latest_versionArray<ArtifactInfo>

Get artifacts for the latest version, channel and product_name When a partial version is set the results will be filtered before return latest version.

Returns:

  • (Array<ArtifactInfo>)

    Array of info about found artifacts



108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
# File 'lib/mixlib/install/backend/package_router.rb', line 108

def latest_version
  product_versions = if options.partial_version?
                       v = options.product_version
                       partial_version = v.end_with?(".") ? v : v + "."
                       versions.find_all { |ver| extract_version_from_response(ver).start_with?(partial_version) }
                     else
                       versions
                     end

  # Use mixlib versioning to parse and sort versions
  ordered_versions = product_versions.sort_by do |v|
    Mixlib::Versioning.parse(extract_version_from_response(v))
  end.reverse

  version = extract_version_from_response(ordered_versions.first)
  artifacts_for_version(version)
end

#use_commercial_api?Boolean

Returns:

  • (Boolean)


355
356
357
# File 'lib/mixlib/install/backend/package_router.rb', line 355

def use_commercial_api?
  !options.license_id.nil? && !options.license_id.to_s.empty? && !use_trial_api?
end

#use_compat_download_url_endpoint?(platform, platform_version) ⇒ boolean

For some older platform & platform_version combinations we need to use COMPAT_DOWNLOAD_URL_ENDPOINT since these versions have an OpenSSL version that can not verify the ENDPOINT based urls

Returns:

  • (boolean)

    use compat download url endpoint



331
332
333
334
335
336
337
338
# File 'lib/mixlib/install/backend/package_router.rb', line 331

def use_compat_download_url_endpoint?(platform, platform_version)
  case "#{platform}-#{platform_version}"
  when "freebsd-9", "el-5", "solaris2-5.9", "solaris2-5.10"
    true
  else
    false
  end
end

#use_licensed_api?Boolean

Returns:

  • (Boolean)


359
360
361
# File 'lib/mixlib/install/backend/package_router.rb', line 359

def use_licensed_api?
  use_trial_api? || use_commercial_api?
end

#use_trial_api?Boolean

Returns:

  • (Boolean)


351
352
353
# File 'lib/mixlib/install/backend/package_router.rb', line 351

def use_trial_api?
  !options.license_id.nil? && !options.license_id.to_s.empty? && options.license_id.start_with?("free-", "trial-")
end

#versionsArray<Array<Hash>] Build records for available versions

Get available versions from Artifactory via AQL. Returning the full API response

Returns:

  • (Array<Array<Hash>] Build records for available versions)

    Array<Array<Hash>] Build records for available versions



66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
# File 'lib/mixlib/install/backend/package_router.rb', line 66

def versions
  # Commercial and trial APIs use a different URL structure
  if use_licensed_api?
    # Response is a JSON array of version strings
    version_list = get("/#{options.channel}/#{omnibus_project}/versions/all")
    # Convert to the expected format with properties
    items = version_list.map do |version|
      { "properties" => [{ "key" => "omnibus.version", "value" => version }] }
    end
  else
    items = get("/api/v1/#{options.channel}/#{omnibus_project}/versions")["results"]
  end

  # Circumvent early when there are no product artifacts in a specific channel
  if items.empty?
    raise ArtifactsNotFound, <<-EOF
No artifacts found matching criteria.
  product name: #{options.product_name}
  channel: #{options.channel}
EOF
  end

  # Filter out the partial builds if we are in :unstable channel
  # In other channels we do not need to do this since all builds are
  # always complete. In fact we should not do this since for some arcane
  # builds like Chef Client 10.X we do not have build record created in
  # artifactory.
  if options.channel == :unstable && !use_licensed_api?
    # We check if "artifacts" field contains something since it is only
    # populated with the build record if "artifact.module.build" exists.
    items.reject! { |i| i["artifacts"].nil? }
  end

  items
end